/*
   BaseProgressIndicator - base class for all progress indicators

   Copyright (C) 2007 Matthias Reichl <hias@horus.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "AtariDebug.h"
#include "BaseProgressIndicator.h"

BaseProgressIndicator::BaseProgressIndicator(size_t end_pos, size_t steps)
	: fEndPos(end_pos),
	  fSteps(steps),
	  fState(eNotRunning),
          fLastStep(0),
          fCurrentStep(0)
{ }

BaseProgressIndicator::~BaseProgressIndicator()
{ }

void BaseProgressIndicator::UpdateIndicator(size_t current_pos)
{
	fLastStep = fCurrentStep;
	switch (fState) {
	case eNotRunning:
		fState = eInitialOutput;
		break;
	case eInitialOutput:
		fState = eRunning;
		break;
	case eRunning:
		break;
	}

	//if (fEndPos < fSteps * 10) {
		fCurrentStep = (current_pos * fSteps + (fEndPos >> 1)) / fEndPos;
	//} else {
	//	size_t one_step = fEndPos / fSteps;
	//	fCurrentStep = (current_pos + (one_step >> 1)) / one_step;
	//}
}

bool BaseProgressIndicator::NeedErase() const
{
	return fState == eRunning;
}

bool BaseProgressIndicator::NeedUpdate() const
{
	switch (fState) {
	case eRunning:
		return fLastStep != fCurrentStep;
	case eInitialOutput:
		return true;
	case eNotRunning:
		return false;
	}
	Assert(false);
	return false;
}

bool BaseProgressIndicator::IsRunning() const
{
	return fState != eNotRunning;
}

void BaseProgressIndicator::AbortIndicator()
{
	fState = eNotRunning;
}
