#ifndef BASEPROGRESSINIDICATOR_H
#define BASEPROGRESSINIDICATOR_H

/*
   BaseProgressIndicator - base class for all progress indicators

   Copyright (C) 2007 Matthias Reichl <hias@horus.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <sys/types.h>
#include "RefCounted.h"
#include "RCPtr.h"

class BaseProgressIndicator : public RefCounted {
public:
	BaseProgressIndicator(size_t end_pos, size_t steps=100);
	virtual ~BaseProgressIndicator();

	virtual void DisplayProgress(size_t current_pos) = 0;
	virtual void FinishProgress() = 0;

protected:
	void UpdateIndicator(size_t current_pos);
	void AbortIndicator();

	bool NeedErase() const;
	bool NeedUpdate() const;
	bool IsRunning() const;
	inline size_t GetCurrentStep() const;
private:
	size_t fEndPos;
	size_t fSteps;

	enum EState {
		eNotRunning,
		eInitialOutput,
		eRunning
	};
	EState fState;

	size_t fLastStep;
	size_t fCurrentStep;
};

inline size_t BaseProgressIndicator::GetCurrentStep() const
{
	return fCurrentStep;
}

#endif
