#ifndef MYIDEPARTITIONINFOCHS_H
#define MYIDEPARTITIONINFOCHS_H

/*
   MyidePartitionInfoCHS - infos for (old) CHS partition table

   Copyright (C) 2008 Matthias Reichl <hias@horus.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "MyidePartitionInfo.h"

class MyidePartitionInfoCHS : public MyidePartitionInfo {
public:
	MyidePartitionInfoCHS();
	virtual ~MyidePartitionInfoCHS();

	virtual bool ParsePartitionInfo(const unsigned char* buf);
	virtual bool CreatePartitionInfo(unsigned char* buf) const;
	
	virtual bool IsChsPartitionTable() const;

	// partition functions
	virtual void PrintPartitionTable() const;

	virtual bool IsActivePartition(unsigned int num) const;
	virtual unsigned int GetPartitionSize(unsigned int num) const;
	virtual EDensity GetPartitionDensity(unsigned int num) const;

	virtual bool IsValidPartitionSector(unsigned int partno, unsigned int secno) const;

	virtual unsigned int GetPartitionSectorNumber(unsigned int partno, unsigned int sector) const;

	// image functions

	virtual bool ImageSpaceEnabled() const;
	virtual unsigned int GetNumberOfImageSlots() const;
	virtual unsigned int GetSectorsPerImageSlot() const;

	virtual bool IsValidImageSector(unsigned int secno) const;

	virtual unsigned int GetImageSectorNumber(unsigned int imageno, unsigned int sector) const;
	virtual unsigned int GetImageInfoSectorNumber(unsigned int imageno) const;

	virtual bool IsValidSectorLength(unsigned int sector, EDensity density, unsigned int len) const;

	inline unsigned int GetNumberOfSectorsPerCylinder() const;

private:
	void UpdateDriveBits();
	
        // physical drive settings
        unsigned int fCylinders;
        unsigned int fHeads;
        unsigned int fSectors;

        unsigned int fSectorsPerCylinder;

        unsigned int fNumberOfPartitions;

	typedef struct {
		unsigned int DriveNumber;
		unsigned int StartingCylinder;
		unsigned int EndingCylinder;
		unsigned int MaxSectors;
		EDensity Density;
		bool IsActive;
	} CHSPartition;
        CHSPartition fPartitionInfo[eMaxPartitions];

        unsigned int fImageSpaceCylinder;
        unsigned int fCylindersPerImageSlot;

};

inline unsigned int MyidePartitionInfoCHS::GetNumberOfSectorsPerCylinder() const
{
	return fSectorsPerCylinder;
}

#endif
