#ifndef MYIDEPARTITIONINFOLBA_H
#define MYIDEPARTITIONINFOLBA_H

/*
   MyidePartitionInfoLBA - infos for (new) LBA partition table

   Copyright (C) 2008 Matthias Reichl <hias@horus.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "MyidePartitionInfo.h"

class MyidePartitionInfoLBA : public MyidePartitionInfo {
public:
	MyidePartitionInfoLBA();
	virtual ~MyidePartitionInfoLBA();

	virtual bool ParsePartitionInfo(const unsigned char* buf);
	virtual bool CreatePartitionInfo(unsigned char* buf) const;

	virtual bool IsLbaPartitionTable() const;

	virtual void PrintPartitionTable() const;

	// partition functions	
	virtual bool IsActivePartition(unsigned int num) const;
	virtual unsigned int GetPartitionSize(unsigned int num) const;
	virtual EDensity GetPartitionDensity(unsigned int num) const;

	virtual bool IsValidPartitionSector(unsigned int partno, unsigned int secno) const;

	virtual unsigned int GetPartitionSectorNumber(unsigned int partno, unsigned int sector) const;

	// image functions	
	virtual bool ImageSpaceEnabled() const;
	virtual unsigned int GetNumberOfImageSlots() const;
	virtual unsigned int GetSectorsPerImageSlot() const;

	virtual bool IsValidImageSector(unsigned int secno) const;

	// absolute
	virtual unsigned int GetImageSectorNumber(unsigned int imageno, unsigned int sector) const;
	virtual unsigned int GetImageInfoSectorNumber(unsigned int imageno) const;


	virtual bool IsValidSectorLength(unsigned int sector, EDensity density, unsigned int len) const;

	// setup partition table functions
	bool SetPartition(unsigned int partition, bool active, unsigned int driveno, unsigned int start, unsigned int sectors, EDensity density);
	bool SetImageSpace(unsigned int start, unsigned int num_slots);
	void SetOptionByte(unsigned char byte);

private:
	void UpdateDriveBits();

	enum { eSectorsPerImageSlot = 1041 };

	// offsets in partition table
	enum {
		ePartitionOffset = 8,
		eImageSlotOffset = 72
	};

	// offsets within global information block
	enum {
		eOfsTotalLBASize = 0,		// 4 bytes
		eOfsLBAIdentification = 4,	// must be $51
		eOfsDriveBits = 5,
		eOfsOptionByte = 6
	};

	// partition block size
	enum {
		ePartitionBlockSize = 8
	};
	// offsets within partition information block
	enum {
		eOfsPartitionDriveDensity = 0,
		eOfsPartitionStart = 1,		// 4 bytes
		eOfsPartitionSize = 5		// 3 bytes
	};
	// offsets within image slot information block
	enum {
		eOfsImageSlotStart = 0,		// 4 bytes
		eOfsImageSlotMax = 4,		// 2 bytes: number of image slots
	};

	enum { ePartSD = 0x00, ePartDD = 0x20, ePartQD = 0x80 };
	
        // physical drive settings
	typedef struct {
		unsigned int DriveNumber;
		unsigned int StartingLBA;
		unsigned int MaxSectors;
		EDensity Density;
		bool IsActive;
	} LBAPartition;
        LBAPartition fPartitionInfo[eMaxPartitions];

	unsigned int fTotalLBASectors;

        unsigned int fImageSpaceStartLBA;
        unsigned int fImageSpaceSlots;

};


#endif
